@@ -6,11 +6,12 @@ import com.unciv.logic.battle.Battle
 import com.unciv.logic.battle.GreatGeneralImplementation
 import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.city.City
-import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitAction
+import com.unciv.models.UnitActionType
+import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
@@ -18,32 +19,6 @@ import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsReligion
 
 object SpecificUnitAutomation {
 
-    private fun hasWorkableSeaResource(tile: Tile, civInfo: Civilization): Boolean =
-            tile.isWater && tile.improvement == null && tile.hasViewableResource(civInfo)
-
-    fun automateWorkBoats(unit: MapUnit) {
-        val closestReachableResource = unit.civ.cities.asSequence()
-                .flatMap { city -> city.getWorkableTiles() }
-                .filter {
-                    hasWorkableSeaResource(it, unit.civ)
-                            && (unit.currentTile == it || unit.movement.canMoveTo(it))
-                }
-                .sortedBy { it.aerialDistanceTo(unit.currentTile) }
-                .firstOrNull { unit.movement.canReach(it) }
-
-        when (closestReachableResource) {
-            null -> UnitAutomation.tryExplore(unit)
-            else -> {
-                unit.movement.headTowards(closestReachableResource)
-
-                // could be either fishing boats or oil well
-                val isImprovable = closestReachableResource.tileResource.getImprovements().any()
-                if (isImprovable && unit.currentTile == closestReachableResource)
-                    UnitActions.getWaterImprovementAction(unit)?.action?.invoke()
-            }
-        }
-    }
-
     fun automateGreatGeneral(unit: MapUnit): Boolean {
         //try to follow nearby units. Do not garrison in city if possible
         val maxAffectedTroopsTile = GreatGeneralImplementation.getBestAffectedTroopsTile(unit)
@@ -57,11 +32,11 @@ object SpecificUnitAutomation {
         // try to revenge and capture their tiles
         val enemyCities = unit.civ.getKnownCivs()
                 .filter { unit.civ.getDiplomacyManager(it).hasModifier(DiplomaticModifiers.StealingTerritory) }
-                .flatMap { it.cities }.asSequence()
+                .flatMap { it.cities }
         // find the suitable tiles (or their neighbours)
         val tileToSteal = enemyCities.flatMap { it.getTiles() } // City tiles
                 .filter { it.neighbors.any { tile -> tile.getOwner() != unit.civ } } // Edge city tiles
-                .flatMap { it.neighbors.asSequence() } // Neighbors of edge city tiles
+                .flatMap { it.neighbors } // Neighbors of edge city tiles
                 .filter {
                     it in unit.civ.viewableTiles // we can see them
                             && it.neighbors.any { tile -> tile.getOwner() == unit.civ }// they are close to our borders
@@ -72,7 +47,7 @@ object SpecificUnitAutomation {
                     // ...also get priorities to steal the most valuable for them
                     val owner = it.getOwner()
                     if (owner != null)
-                        distance - WorkerAutomation.getPriority(it, owner)
+                        distance - owner.getWorkerAutomation().getPriority(it)
                     else distance
                 }
                 .firstOrNull { unit.movement.canReach(it) } // canReach is performance-heavy and always a last resort
@@ -85,7 +60,7 @@ object SpecificUnitAutomation {
         }
 
         // try to build a citadel for defensive purposes
-        if (WorkerAutomation.evaluateFortPlacement(unit.currentTile, unit.civ, true)) {
+        if (unit.civ.getWorkerAutomation().evaluateFortPlacement(unit.currentTile, true)) {
             UnitActions.getImprovementConstructionActions(unit, unit.currentTile).firstOrNull()?.action?.invoke()
             return true
         }
@@ -112,7 +87,7 @@ object SpecificUnitAutomation {
         val tileForCitadel = cityToGarrison.getTilesInDistanceRange(3..4)
             .firstOrNull {
                 reachableTest(it) &&
-                        WorkerAutomation.evaluateFortPlacement(it, unit.civ, true)
+                    unit.civ.getWorkerAutomation().evaluateFortPlacement(it, true)
             }
         if (tileForCitadel == null) {
             unit.movement.headTowards(cityToGarrison)
@@ -124,12 +99,12 @@ object SpecificUnitAutomation {
                 .firstOrNull()?.action?.invoke()
     }
 
-    fun automateSettlerActions(unit: MapUnit) {
+    fun automateSettlerActions(unit: MapUnit, tilesWhereWeWillBeCaptured: Set<Tile>) {
         if (unit.civ.gameInfo.turns == 0) {   // Special case, we want AI to settle in place on turn 1.
             val foundCityAction = UnitActions.getFoundCityAction(unit, unit.getTile())
             // Depending on era and difficulty we might start with more than one settler. In that case settle the one with the best location
             val otherSettlers = unit.civ.units.getCivUnits().filter { it.currentMovement > 0 && it.baseUnit == unit.baseUnit }
-            if(foundCityAction?.action != null &&
+            if (foundCityAction?.action != null &&
                     otherSettlers.none {
                         CityLocationTileRanker.rankTileAsCityCenter(
                             it.getTile(), unit.civ
@@ -143,14 +118,11 @@ object SpecificUnitAutomation {
             }
         }
 
-        if (unit.getTile().militaryUnit == null     // Don't move until you're accompanied by a military unit
-            && !unit.civ.isCityState()          // ..unless you're a city state that was unable to settle its city on turn 1
-            && unit.getDamageFromTerrain() < unit.health) return    // Also make sure we won't die waiting
-
         // It's possible that we'll see a tile "over the sea" that's better than the tiles close by, but that's not a reason to abandon the close tiles!
         // Also this lead to some routing problems, see https://github.com/yairm210/Unciv/issues/3653
         val bestCityLocation: Tile? =
                 CityLocationTileRanker.getBestTilesToFoundCity(unit).firstOrNull {
+                    if (it.first in tilesWhereWeWillBeCaptured) return@firstOrNull false
                     val pathSize = unit.movement.getShortestPath(it.first).size
                     return@firstOrNull pathSize in 1..3
                 }?.first
@@ -167,7 +139,7 @@ object SpecificUnitAutomation {
             if (frontierCity != null && getFrontierScore(frontierCity) > 0  && unit.movement.canReach(frontierCity.getCenterTile()))
                 unit.movement.headTowards(frontierCity.getCenterTile())
             if (UnitAutomation.tryExplore(unit)) return // try to find new areas
-            UnitAutomation.wander(unit) // go around aimlessly
+            UnitAutomation.wander(unit, tilesToAvoid = tilesWhereWeWillBeCaptured) // go around aimlessly
             return
         }
 
@@ -183,13 +155,14 @@ object SpecificUnitAutomation {
             foundCityAction.action.invoke()
     }
 
-    fun automateImprovementPlacer(unit: MapUnit) {
-        val improvementBuildingUniques = unit.getMatchingUniques(UniqueType.ConstructImprovementConsumingUnit) +
-                unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
+    /** @return whether there was any progress in placing the improvement. A return value of `false`
+     * can be interpreted as: the unit doesn't know where to place the improvement or is stuck. */
+    fun automateImprovementPlacer(unit: MapUnit) : Boolean {
+        val improvementBuildingUniques = unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
 
         val improvementName = improvementBuildingUniques.first().params[0]
         val improvement = unit.civ.gameInfo.ruleset.tileImprovements[improvementName]
-            ?: return
+            ?: return false
         val relatedStat = improvement.maxByOrNull { it.value }?.key ?: Stat.Culture
 
         val citiesByStatBoost = unit.civ.cities.sortedByDescending {
@@ -209,9 +182,18 @@ object SpecificUnitAutomation {
 
             if (pathToCity.isEmpty()) continue
             if (pathToCity.size > 2 && unit.getTile().getCity() != city) {
-                if (unit.getTile().militaryUnit == null) return // Don't move until you're accompanied by a military unit
+                // Radius 5 is quite arbitrary. Few units have such a high movement radius although
+                // streets might modify it. Also there might be invisible units, so this is just an
+                // approximation for relative safety and simplicity.
+                val enemyUnitsNearby = unit.getTile().getTilesInDistance(5).any { tileNearby ->
+                    tileNearby.getUnits().any { unitOnTileNearby ->
+                        unitOnTileNearby.isMilitary() && unitOnTileNearby.civ.isAtWarWith(unit.civ)
+                    }
+                }
+                // Don't move until you're accompanied by a military unit if there are enemies nearby.
+                if (unit.getTile().militaryUnit == null && enemyUnitsNearby) return true
                 unit.movement.headTowards(city.getCenterTile())
-                return
+                return true
             }
 
             // if we got here, we're pretty close, start looking!
@@ -224,14 +206,107 @@ object SpecificUnitAutomation {
                 .firstOrNull { unit.movement.canReach(it) }
                 ?: continue // to another city
 
+            val unitTileBeforeMovement = unit.currentTile
             unit.movement.headTowards(chosenTile)
-            if (unit.currentTile == chosenTile)
+            if (unit.currentTile == chosenTile) {
                 if (unit.currentTile.isPillaged())
                     UnitActions.getRepairAction(unit).invoke()
                 else
-                    UnitActions.getImprovementConstructionActions(unit, unit.currentTile).firstOrNull()?.action?.invoke()
-            return
+                    UnitActions.getImprovementConstructionActions(unit, unit.currentTile)
+                        .firstOrNull()?.action?.invoke()
+                return true
+            }
+            return unitTileBeforeMovement != unit.currentTile
         }
+        // No city needs this improvement.
+        return false
+    }
+
+    /** @return whether there was any progress in conducting the trade mission. A return value of
+     * `false` can be interpreted as: the unit doesn't know where to go or there are no city
+     * states. */
+    fun conductTradeMission(unit: MapUnit): Boolean {
+        val closestCityStateTile =
+                unit.civ.gameInfo.civilizations
+                    .filter {
+                        it != unit.civ
+                            && !unit.civ.isAtWarWith(it)
+                            && it.isCityState()
+                            && it.cities.isNotEmpty()
+                    }
+                    .flatMap { it.cities[0].getTiles() }
+                    .filter { unit.civ.hasExplored(it) }
+                    .mapNotNull { tile ->
+                        val path = unit.movement.getShortestPath(tile)
+                        // 0 is unreachable, 10 is too far away
+                        if (path.size in 1..10) tile to path.size else null
+                    }
+                    .minByOrNull { it.second }?.first
+                    ?: return false
+
+        val conductTradeMissionAction = UnitActions.getUnitActions(unit)
+            .firstOrNull { it.type == UnitActionType.ConductTradeMission }
+        if (conductTradeMissionAction?.action != null) {
+            conductTradeMissionAction.action.invoke()
+            return true
+        }
+
+        val unitTileBeforeMovement = unit.currentTile
+        unit.movement.headTowards(closestCityStateTile)
+
+        return unitTileBeforeMovement != unit.currentTile
+    }
+
+    /**
+     * If there's a city nearby that can construct a wonder, walk there an get it built. Typically I
+     * like to build all wonders in the same city to have the boni accumulate (and it typically ends
+     * up being my capital), but that would need too much logic (e.g. how far away is the capital,
+     * is the wonder likely still available by the time I'm there, is this particular wonder even
+     * buildable in the capital, etc.)
+     *
+     * @return whether there was any progress in speeding up a wonder construction. A return value
+     * of `false` can be interpreted as: the unit doesn't know where to go or is stuck. */
+    fun speedupWonderConstruction(unit: MapUnit): Boolean {
+        val nearbyCityWithAvailableWonders = unit.civ.cities.filter { city ->
+            // Don't speed up construction in small cities. There's a risk the great
+            // engineer can't get it done entirely and then it takes forever for the small
+            // city to finish the rest.
+            city.population.population >= 3 &&
+            (unit.movement.canMoveTo(city.getCenterTile()) || unit.currentTile == city.getCenterTile())
+                    && getWonderThatWouldBenefitFromBeingSpedUp(city) != null
+        }.mapNotNull { city ->
+            val path = unit.movement.getShortestPath(city.getCenterTile())
+            if (path.any() && path.size <= 5) city to path.size else null
+        }.minByOrNull { it.second }
+            ?.first ?: return false
+
+        if (unit.currentTile == nearbyCityWithAvailableWonders.getCenterTile()) {
+            val wonderToHurry =
+                    getWonderThatWouldBenefitFromBeingSpedUp(nearbyCityWithAvailableWonders)!!
+            nearbyCityWithAvailableWonders.cityConstructions.constructionQueue.add(
+                0,
+                wonderToHurry.name
+            )
+            unit.showAdditionalActions = false  // make sure getUnitActions doesn't skip the important ones
+            val unitAction = UnitActions.getUnitActions(unit).firstOrNull {
+                    it.type == UnitActionType.HurryBuilding || it.type == UnitActionType.HurryWonder
+                } ?: return false
+            if (unitAction.action == null) return false
+            unitAction.action.invoke()
+            return true
+        }
+
+        // Walk towards the city.
+        val tileBeforeMoving = unit.getTile()
+        unit.movement.headTowards(nearbyCityWithAvailableWonders.getCenterTile())
+        return tileBeforeMoving != unit.currentTile
+    }
+
+    private fun getWonderThatWouldBenefitFromBeingSpedUp(city: City): Building? {
+        return city.cityConstructions.getBuildableBuildings().filter { building ->
+            building.isWonder && !building.hasUnique(UniqueType.CannotBeHurried)
+                    && city.cityConstructions.turnsToConstruction(building.name) >= 5
+        }.sortedBy { -city.cityConstructions.getRemainingWork(it.name) }.firstOrNull()
     }
 
     fun automateAddInCapital(unit: MapUnit) {
@@ -256,14 +331,14 @@ object SpecificUnitAutomation {
         val city =
             if (ourCitiesWithoutReligion.any())
                 ourCitiesWithoutReligion.minByOrNull { it.getCenterTile().aerialDistanceTo(unit.getTile()) }
-            else unit.civ.gameInfo.getCities().asSequence()
+            else unit.civ.gameInfo.getCities()
                 .filter { it.religion.getMajorityReligion() != unit.civ.religionManager.religion }
                 .filter { it.civ.knows(unit.civ) && !it.civ.isAtWarWith(unit.civ) }
                 .filterNot { it.religion.isProtectedByInquisitor(unit.religion) }
                 .minByOrNull { it.getCenterTile().aerialDistanceTo(unit.getTile()) }
 
         if (city == null) return
-        val destination = city.getTiles().asSequence()
+        val destination = city.getTiles()
             .filter { unit.movement.canMoveTo(it) || it == unit.getTile() }
             .sortedBy { it.aerialDistanceTo(unit.getTile()) }
             .firstOrNull { unit.movement.canReach(it) } ?: return
@@ -317,7 +392,7 @@ object SpecificUnitAutomation {
         if (destination == null) return
 
         if (!unit.movement.canReach(destination)) {
-            destination = destination.neighbors.asSequence()
+            destination = destination.neighbors
                 .filter { unit.movement.canMoveTo(it) || it == unit.getTile() }
                 .sortedBy { it.aerialDistanceTo(unit.currentTile) }
                 .firstOrNull { unit.movement.canReach(it) }
@@ -425,8 +500,7 @@ object SpecificUnitAutomation {
                     && tile.getOwner()!!.isAtWarWith(unit.civ)
                     && tile.getCity()!!.health > tile.getCity()!!.getMaxHealth() / 2
                     && Battle.mayUseNuke(MapUnitCombatant(unit), tile)) {
-                val blastRadius = unit.getMatchingUniques(UniqueType.BlastRadius)
-                    .firstOrNull()?.params?.get(0)?.toInt() ?: 2
+                val blastRadius = unit.getNukeBlastRadius()
                 val tilesInBlastRadius = tile.getTilesInDistance(blastRadius)
                 val civsInBlastRadius = tilesInBlastRadius.mapNotNull { it.getOwner() } +
                         tilesInBlastRadius.mapNotNull { it.getFirstUnit()?.civ }
@@ -472,6 +546,7 @@ object SpecificUnitAutomation {
         for (city in immediatelyReachableCitiesAndCarriers) {
             if (city.getTilesInDistance(unit.getRange())
                             .any {
+                                it.isVisible(unit.civ) &&
                                 BattleHelper.containsAttackableEnemy(
                                     it,
                                     MapUnitCombatant(unit)
