@@ -58,7 +58,8 @@ class WorkerAutomation(
     private val citiesThatNeedConnecting: List<City> by lazy {
         val result = civInfo.cities.asSequence()
             .filter {
-                it.population.population > 3
+                civInfo.getCapital() != null
+                    && it.population.population > 3
                         && !it.isCapital() && !it.isBeingRazed // Cities being razed should not be connected.
                         && !it.cityStats.isConnectedToCapital(bestRoadAvailable)
             }.sortedBy {
@@ -107,21 +108,6 @@ class WorkerAutomation(
     ///////////////////////////////////////// Helpers /////////////////////////////////////////
 
     companion object {
-        /** Maps to instance [WorkerAutomation.automateWorkerAction] knowing only the MapUnit */
-        fun automateWorkerAction(unit: MapUnit) {
-            unit.civ.getWorkerAutomation().automateWorkerAction(unit)
-        }
-
-        /** Convenience shortcut supports old calling syntax for [WorkerAutomation.getPriority] */
-        fun getPriority(tile: Tile, civInfo: Civilization): Int {
-            return civInfo.getWorkerAutomation().getPriority(tile)
-        }
-
-        /** Convenience shortcut supports old calling syntax for [WorkerAutomation.evaluateFortPlacement] */
-        fun evaluateFortPlacement(tile: Tile, civInfo: Civilization, isCitadel: Boolean): Boolean {
-            return civInfo.getWorkerAutomation().evaluateFortPlacement(tile, isCitadel)
-        }
-
         /** For console logging only */
         private fun MapUnit.label() = toString() + " " + getTile().position.toString()
     }
@@ -131,11 +117,11 @@ class WorkerAutomation(
     /**
      * Automate one Worker - decide what to do and where, move, start or continue work.
      */
-    fun automateWorkerAction(unit: MapUnit) {
+    fun automateWorkerAction(unit: MapUnit, tilesWhereWeWillBeCaptured: Set<Tile>) {
         val currentTile = unit.getTile()
-        val tileToWork = findTileToWork(unit)
+        val tileToWork = findTileToWork(unit, tilesWhereWeWillBeCaptured)
 
-        if (getPriority(tileToWork, civInfo) < 3) { // building roads is more important
+        if (getPriority(tileToWork) < 3) { // building roads is more important
             if (tryConnectingCities(unit)) return
         }
 
@@ -143,20 +129,27 @@ class WorkerAutomation(
             debug("WorkerAutomation: %s -> head towards %s", unit.label(), tileToWork)
             val reachedTile = unit.movement.headTowards(tileToWork)
             if (reachedTile != currentTile) unit.doAction() // otherwise, we get a situation where the worker is automated, so it tries to move but doesn't, then tries to automate, then move, etc, forever. Stack overflow exception!
+
+            //If we have reached a fort tile that is in progress and shouldn't be there, cancel it.
+            if (reachedTile == tileToWork && reachedTile.improvementInProgress == Constants.fort && !evaluateFortSuroundings(currentTile, false)) {
+                debug("Replacing fort in progress with new improvement")
+                reachedTile.stopWorkingOnImprovement()
+            }
+
             // If there's move still left, perform action
             // Unit may stop due to Enemy Unit within walking range during doAction() call
             if (unit.currentMovement > 0 && reachedTile == tileToWork) {
                 if (reachedTile.isPillaged()) {
-                    debug("WorkerAutomation: ${unit.label()} -> repairs $currentTile")
+                    debug("WorkerAutomation: ${unit.label()} -> repairs $reachedTile")
                     UnitActions.getRepairAction(unit).invoke()
                     return
                 }
-                if (currentTile.improvementInProgress == null && currentTile.isLand
-                        && tileCanBeImproved(unit, currentTile)
+                if (reachedTile.improvementInProgress == null && reachedTile.isLand
+                        && tileCanBeImproved(unit, reachedTile)
                 ) {
-                    debug("WorkerAutomation: ${unit.label()} -> start improving $currentTile")
-                    return currentTile.startWorkingOnImprovement(
-                        chooseImprovement(unit, currentTile)!!, civInfo, unit
+                    debug("WorkerAutomation: ${unit.label()} -> start improving $reachedTile")
+                    return reachedTile.startWorkingOnImprovement(
+                        chooseImprovement(unit, reachedTile)!!, civInfo, unit
                     )
                 }
             }
@@ -169,13 +162,18 @@ class WorkerAutomation(
             return
         }
 
-        if (currentTile.improvementInProgress == null && currentTile.isLand
-            && tileCanBeImproved(unit, currentTile)) {
+        if (currentTile.improvementInProgress == null && tileCanBeImproved(unit, currentTile)) {
             debug("WorkerAutomation: ${unit.label()} -> start improving $currentTile")
             return currentTile.startWorkingOnImprovement(chooseImprovement(unit, currentTile)!!, civInfo, unit)
         }
 
         if (currentTile.improvementInProgress != null) return // we're working!
+
+        if (unit.cache.hasUniqueToCreateWaterImprovements) {
+            // Support Alpha Frontier-Style Workers that _also_ have the "May create improvements on water resources" unique
+            if (automateWorkBoats(unit)) return
+        }
+
         if (tryConnectingCities(unit)) return //nothing to do, try again to connect cities
 
         val citiesToNumberOfUnimprovedTiles = HashMap<String, Int>()
@@ -201,7 +199,7 @@ class WorkerAutomation(
 
         // Idle CS units should wander so they don't obstruct players so much
         if (unit.civ.isCityState())
-            wander(unit, stayInTerritory = true)
+            wander(unit, stayInTerritory = true, tilesToAvoid = tilesWhereWeWillBeCaptured)
     }
 
     /**
@@ -220,7 +218,7 @@ class WorkerAutomation(
         }
         if (candidateCities.none()) return false // do nothing.
 
-        val isCandidateTilePredicate = { it: Tile -> it.isLand && unit.movement.canPassThrough(it) }
+        val isCandidateTilePredicate: (Tile) -> Boolean = { it.isLand && unit.movement.canPassThrough(it) }
         val currentTile = unit.getTile()
         val cityTilesToSeek = ArrayList(tilesOfConnectedCities.sortedBy { it.aerialDistanceTo(currentTile) })
 
@@ -279,27 +277,42 @@ class WorkerAutomation(
      * Looks for a worthwhile tile to improve
      * @return The current tile if no tile to work was found
      */
-    private fun findTileToWork(unit: MapUnit): Tile {
+    private fun findTileToWork(unit: MapUnit, tilesToAvoid: Set<Tile>): Tile {
         val currentTile = unit.getTile()
-        val workableTiles = currentTile.getTilesInDistance(4)
+        val workableTilesCenterFirst = currentTile.getTilesInDistance(4)
                 .filter {
-                    (it.civilianUnit == null || it == currentTile)
-                            && (it.owningCity == null || it.getOwner()==civInfo)
-                            && getPriority(it) > 1
-                            && it.getTilesInDistance(2)  // don't work in range of enemy cities
+                    it !in tilesToAvoid
+                    && (it.civilianUnit == null || it == currentTile)
+                    && (it.owningCity == null || it.getOwner() == civInfo)
+                    && getPriority(it) > 1
+                    && it.getTilesInDistance(2)  // don't work in range of enemy cities
                         .none { tile -> tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(civInfo) }
-                            && it.getTilesInDistance(3)  // don't work in range of enemy units
+                    && it.getTilesInDistance(3)  // don't work in range of enemy units
                         .none { tile -> tile.militaryUnit != null && tile.militaryUnit!!.civ.isAtWarWith(civInfo)}
                 }
-                .sortedByDescending { getPriority(it) }
+
+        // Carthage can move through mountains, special case
+        // If there are non-damage dealing tiles available, move to the best of those, otherwise move to the best damage dealing tile
+        val workableTilesPrioritized = workableTilesCenterFirst
+                .sortedWith(
+                    compareBy<Tile> { unit.getDamageFromTerrain(it) > 0 } // Sort on Boolean puts false first
+                        .thenByDescending { getPriority(it) }
+                )
 
         // These are the expensive calculations (tileCanBeImproved, canReach), so we only apply these filters after everything else it done.
-        val selectedTile = workableTiles.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+        val selectedTile = workableTilesPrioritized
+            .firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
+            ?: return currentTile
 
-        return if (selectedTile != null
-                && ((!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
-                    || !workableTiles.contains(currentTile) // current tile is unworkable by city
-                    || getPriority(selectedTile) > getPriority(currentTile)))  // current tile is less important
+        // Note: workableTiles is a Sequence, and we oiginally used workableTiles.contains for the second
+        // test, which looks like a second potentially deep iteration of it, after just being iterated
+        // for selectedTile. But TileMap.getTilesInDistanceRange iterates its range forward, meaning
+        // currentTile is always the very first entry of the _unsorted_ Sequence - if it is still
+        // contained at all and not dropped by the filters - which is the point here.
+        return if ( currentTile == selectedTile  // No choice
+                || (!tileCanBeImproved(unit, currentTile) && !currentTile.isPillaged()) // current tile is unimprovable
+                || workableTilesCenterFirst.firstOrNull() != currentTile  // current tile is unworkable by city
+                || getPriority(selectedTile) > getPriority(currentTile))  // current tile is less important
             selectedTile
         else currentTile
     }
@@ -309,40 +322,48 @@ class WorkerAutomation(
      * (but does not check whether the ruleset contains any unit capable of it)
      */
     private fun tileCanBeImproved(unit: MapUnit, tile: Tile): Boolean {
-        if (!tile.isLand || tile.isImpassible() || tile.isCityCenter())
-            return false
+        //todo This is wrong but works for Alpha Frontier, because the unit has both:
+        // It should test for the build over time ability, but this tests the create and die ability
+        if (!tile.isLand && !unit.cache.hasUniqueToCreateWaterImprovements) return false
+        // Allow outlandish mods having non-road improvements on Mountains
+        if (tile.isImpassible() && !unit.cache.canPassThroughImpassableTiles) return false
+        if (tile.isCityCenter()) return false
+
         val city = tile.getCity()
         if (city == null || city.civ != civInfo)
             return false
+
         if (!city.tilesInRange.contains(tile)
                 && !tile.hasViewableResource(civInfo)
                 && civInfo.cities.none { it.getCenterTile().aerialDistanceTo(tile) <= 3 })
             return false // unworkable tile
 
-        val junkImprovement = tile.getTileImprovement()?.hasUnique(UniqueType.AutomatedWorkersWillReplace)
-        if (tile.improvement != null && junkImprovement == false
+        //If the tile is a junk improvement or a fort placed in a bad location.
+        val junkImprovement = tile.getTileImprovement()?.hasUnique(UniqueType.AutomatedWorkersWillReplace) == true
+            || (tile.improvement == Constants.fort && !evaluateFortSuroundings(tile, false) && !civInfo.isHuman())
+
+        if (tile.improvement != null && !junkImprovement
                 && !UncivGame.Current.settings.automatedWorkersReplaceImprovements
                 && unit.civ.isHuman())
             return false
 
-
-
-        if (tile.improvement == null || junkImprovement == true) {
+        if (tile.improvement == null || junkImprovement) {
             if (tile.improvementInProgress != null && unit.canBuildImprovement(tile.getTileImprovementInProgress()!!, tile)) return true
             val chosenImprovement = chooseImprovement(unit, tile)
             if (chosenImprovement != null && tile.improvementFunctions.canBuildImprovement(chosenImprovement, civInfo) && unit.canBuildImprovement(chosenImprovement, tile)) return true
         } else if (!tile.containsGreatImprovement() && tile.hasViewableResource(civInfo)
-            && tile.tileResource.isImprovedBy(tile.improvement!!)
+            && !tile.tileResource.isImprovedBy(tile.improvement!!)
             && (chooseImprovement(unit, tile) // if the chosen improvement is not null and buildable
                 .let { it != null && tile.improvementFunctions.canBuildImprovement(it, civInfo) && unit.canBuildImprovement(it, tile)}))
             return true
+
         return false // couldn't find anything to construct here
     }
 
     /**
      * Calculate a priority for improving a tile
      */
-    private fun getPriority(tile: Tile): Int {
+    fun getPriority(tile: Tile): Int {
         var priority = 0
         if (tile.getOwner() == civInfo) {
             priority += 2
@@ -370,35 +391,34 @@ class WorkerAutomation(
         if (potentialTileImprovements.isEmpty()) return null
 
         val cityUniqueCaches = HashMap<City, LocalUniqueCache>()
-        fun getRankingWithImprovement(improvementName: String): Float {
+        fun getImprovementRanking(improvementName: String): Float {
             val improvement = ruleSet.tileImprovements[improvementName]!!
             val city = tile.getCity()
             val cache =
                     if (city == null) LocalUniqueCache(false)
                     else cityUniqueCaches.getOrPut(city) { LocalUniqueCache() }
-            val stats = tile.stats.getImprovementStats(improvement, civInfo, tile.getCity(), cache)
+            val stats = tile.stats.getStatDiffForImprovement(improvement, civInfo, tile.getCity(), cache)
             return Automation.rankStatsValue(stats, unit.civ)
         }
 
         val bestBuildableImprovement = potentialTileImprovements.values.asSequence()
-            .map { Pair(it, getRankingWithImprovement(it.name)) }
+            .map { Pair(it, getImprovementRanking(it.name)) }
             .filter { it.second > 0f }
             .maxByOrNull { it.second }?.first
 
         val lastTerrain = tile.lastTerrain
 
-        fun isUnbuildableAndRemovable(terrain: Terrain): Boolean = terrain.unbuildable
-                && ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
-
+        fun isRemovable(terrain: Terrain): Boolean = ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
 
         val improvementStringForResource: String? = when {
             tile.resource == null || !tile.hasViewableResource(civInfo) -> null
             tile.terrainFeatures.isNotEmpty()
-                    && isUnbuildableAndRemovable(lastTerrain)
-                    && !tile.providesResources(civInfo)
-                    && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
+                && lastTerrain.unbuildable
+                && isRemovable(lastTerrain)
+                && !tile.providesResources(civInfo)
+                && !isResourceImprovementAllowedOnFeature(tile, potentialTileImprovements) -> Constants.remove + lastTerrain.name
             else -> tile.tileResource.getImprovements().filter { it in potentialTileImprovements || it==tile.improvement }
-                .maxByOrNull { getRankingWithImprovement(it) }
+                .maxByOrNull { getImprovementRanking(it) }
         }
 
         // After gathering all the data, we conduct the hierarchy in one place
@@ -409,12 +429,14 @@ class WorkerAutomation(
             tile.resource != null && tile.tileResource.getImprovements().any() -> return null
             bestBuildableImprovement == null -> null
 
-            tile.improvement!=null && getRankingWithImprovement(tile.improvement!!) > getRankingWithImprovement(bestBuildableImprovement.name)
+            tile.improvement != null &&
+                    getImprovementRanking(tile.improvement!!) > getImprovementRanking(bestBuildableImprovement.name)
                 -> null // What we have is better, even if it's pillaged we should repair it
 
             lastTerrain.let {
-                isUnbuildableAndRemovable(it) &&
-                        (Automation.rankStatsValue(it, civInfo) < 0 || it.hasUnique(UniqueType.NullifyYields) )
+                isRemovable(it) &&
+                    (Automation.rankStatsValue(it, civInfo) < 0
+                        || it.hasUnique(UniqueType.NullifyYields))
             } -> Constants.remove + lastTerrain.name
 
             else -> bestBuildableImprovement.name
@@ -439,7 +461,7 @@ class WorkerAutomation(
     }
 
     /**
-     * Checks whether a given tile allows a Fort and whether a Fort may be undesirable (without checking surroundings).
+     * Checks whether a given tile allows a Fort and whether a Fort may be undesirable (without checking surroundings or if there is a fort already on the tile).
      *
      * -> Checks: city, already built, resource, great improvements.
      * Used only in [evaluateFortPlacement].
@@ -448,7 +470,6 @@ class WorkerAutomation(
         //todo Should this not also check impassable and the fort improvement's terrainsCanBeBuiltOn/uniques?
         if (tile.isCityCenter() // don't build fort in the city
             || !tile.isLand // don't build fort in the water
-            || tile.improvement == Constants.fort // don't build fort if it is already here
             || tile.hasViewableResource(civInfo) // don't build on resource tiles
             || tile.containsGreatImprovement() // don't build on great improvements (including citadel)
         ) return false
@@ -458,10 +479,12 @@ class WorkerAutomation(
 
     /**
      * Do we want a Fort [here][tile] considering surroundings?
+     * (but does not check if if there is already a fort here)
+     *
      * @param  isCitadel Controls within borders check - true also allows 1 tile outside borders
-     * @return Yes please build a Fort here
+     * @return Yes the location is good for a Fort here
      */
-    private fun evaluateFortPlacement(tile: Tile, isCitadel: Boolean): Boolean {
+    private fun evaluateFortSuroundings(tile: Tile, isCitadel: Boolean): Boolean {
         //todo Is the Citadel code dead anyway? If not - why does the nearestTiles check not respect the param?
 
         // build on our land only
@@ -526,4 +549,54 @@ class WorkerAutomation(
         return distanceBetweenCities + 2 > distanceToEnemy + distanceToOurCity
     }
 
+    /**
+     * Do we want to build a Fort [here][tile] considering surroundings?
+     *
+     * @param  isCitadel Controls within borders check - true also allows 1 tile outside borders
+     * @return Yes the location is good for a Fort here
+     */
+    fun evaluateFortPlacement(tile: Tile, isCitadel: Boolean): Boolean {
+        return tile.improvement != Constants.fort // don't build fort if it is already here
+            && evaluateFortSuroundings(tile,isCitadel)
+    }
+
+    private fun hasWorkableSeaResource(tile: Tile, civInfo: Civilization): Boolean =
+        tile.isWater && tile.improvement == null && tile.hasViewableResource(civInfo)
+
+    /** Try improving a Water Resource
+     *
+     *  No logic to avoid capture by enemies yet!
+     *
+     *  @return Whether any progress was made (improved a tile or at least moved towards an opportunity)
+     */
+    fun automateWorkBoats(unit: MapUnit): Boolean {
+        val closestReachableResource = unit.civ.cities.asSequence()
+            .flatMap { city -> city.getWorkableTiles() }
+            .filter {
+                hasWorkableSeaResource(it, unit.civ)
+                    && (unit.currentTile == it || unit.movement.canMoveTo(it))
+            }
+            .sortedBy { it.aerialDistanceTo(unit.currentTile) }
+            .firstOrNull { unit.movement.canReach(it) }
+            ?: return false
+
+        // could be either fishing boats or oil well
+        val isImprovable = closestReachableResource.tileResource.getImprovements().any()
+        if (!isImprovable) return false
+
+        unit.movement.headTowards(closestReachableResource)
+        if (unit.currentTile != closestReachableResource) return true // moving counts as progress
+
+        // We know we have the CreateWaterImprovements, but not whether
+        // all conditionals succeed with a current StateForConditionals(civ, unit)
+        // todo: Not necessarily the optimal flow: Be optimistic and head towards,
+        //       then when arrived and the conditionals say "no" do something else instead?
+        val action = UnitActions.getWaterImprovementAction(unit)
+            ?: return false
+
+        // If action.action is null that means only transient reasons prevent the improvement -
+        // report progress and hope next run it will work.
+        action.action?.invoke()
+        return true
+    }
 }
