@@ -5,6 +5,7 @@ import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.automation.civilization.NextTurnAutomation
 import com.unciv.logic.automation.unit.AttackableTile
+import com.unciv.logic.automation.unit.SpecificUnitAutomation
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.AlertType
 import com.unciv.logic.civilization.Civilization
@@ -27,10 +28,12 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.ui.components.extensions.toPercent
+import com.unciv.ui.screens.worldscreen.bottombar.BattleTable
 import com.unciv.utils.debug
-import java.util.*
 import kotlin.math.max
 import kotlin.math.min
+import kotlin.math.ulp
+import kotlin.random.Random
 
 /**
  * Damage calculations according to civ v wiki and https://steamcommunity.com/sharedfiles/filedetails/?id=170194443
@@ -40,6 +43,8 @@ object Battle {
     /**
      * Moves [attacker] to [attackableTile], handles siege setup then attacks if still possible
      * (by calling [attack] or [NUKE]). Does _not_ play the attack sound!
+     *
+     * Currently not used by UI, only by automation via [BattleHelper.tryAttackNearbyEnemy][com.unciv.logic.automation.unit.BattleHelper.tryAttackNearbyEnemy]
      */
     fun moveAndAttack(attacker: ICombatant, attackableTile: AttackableTile) {
         if (!movePreparingAttack(attacker, attackableTile)) return
@@ -83,14 +88,16 @@ object Battle {
     /**
      * This is meant to be called only after all prerequisite checks have been done.
      */
-    fun attackOrNuke(attacker: ICombatant, attackableTile: AttackableTile) {
-        if (attacker is MapUnitCombatant && attacker.unit.baseUnit.isNuclearWeapon())
+    fun attackOrNuke(attacker: ICombatant, attackableTile: AttackableTile): DamageDealt {
+        return if (attacker is MapUnitCombatant && attacker.unit.baseUnit.isNuclearWeapon()) {
             NUKE(attacker, attackableTile.tileToAttack)
-        else
+            DamageDealt.None
+        } else {
             attack(attacker, getMapCombatantOfTile(attackableTile.tileToAttack)!!)
+        }
     }
 
-    fun attack(attacker: ICombatant, defender: ICombatant) {
+    fun attack(attacker: ICombatant, defender: ICombatant): DamageDealt {
         debug("%s %s attacked %s %s", attacker.getCivInfo().civName, attacker.getName(), defender.getCivInfo().civName, defender.getName())
         val attackedTile = defender.getTile()
         if (attacker is MapUnitCombatant) {
@@ -103,10 +110,11 @@ object Battle {
             ))
         }
 
+        val interceptDamage: DamageDealt
         if (attacker is MapUnitCombatant && attacker.unit.baseUnit.isAirUnit()) {
-            tryInterceptAirAttack(attacker, attackedTile, defender.getCivInfo(), defender)
-            if (attacker.isDefeated()) return
-        }
+            interceptDamage = tryInterceptAirAttack(attacker, attackedTile, defender.getCivInfo(), defender)
+            if (attacker.isDefeated()) return interceptDamage
+        } else interceptDamage = DamageDealt.None
 
         // Withdraw from melee ability
         if (attacker is MapUnitCombatant && attacker.isMelee() && defender is MapUnitCombatant) {
@@ -114,7 +122,8 @@ object Battle {
             val combinedProbabilityToStayPut = withdrawUniques.fold(100) { probabilityToStayPut, unique -> probabilityToStayPut * (100-unique.params[0].toInt()) / 100 }
             val baseWithdrawChance = 100 - combinedProbabilityToStayPut
             // If a mod allows multiple withdraw properties, they stack multiplicatively
-            if (baseWithdrawChance != 0 && doWithdrawFromMeleeAbility(attacker, defender, baseWithdrawChance)) return
+            if (baseWithdrawChance != 0 && doWithdrawFromMeleeAbility(attacker, defender, baseWithdrawChance))
+                return DamageDealt.None
         }
 
         val isAlreadyDefeatedCity = defender is CityCombatant && defender.isDefeated()
@@ -158,7 +167,6 @@ object Battle {
                     UniqueTriggerActivation.triggerUnitwideUnique(unique, ourUnit.unit, triggerNotificationText = "due to our [${ourUnit.getName()}] defeating a [${enemy.getName()}]")
         }
 
-
         // Add culture when defeating a barbarian when Honor policy is adopted, gold from enemy killed when honor is complete
         // or any enemy military unit with Sacrificial captives unique (can be either attacker or defender!)
         if (defender.isDefeated() && defender is MapUnitCombatant && !defender.unit.isCivilian()) {
@@ -197,6 +205,8 @@ object Battle {
                 .firstOrNull { it.text == "Your city [${attacker.getName()}] can bombard the enemy!" }
             attacker.getCivInfo().notifications.remove(cityCanBombardNotification)
         }
+
+        return damageDealt + interceptDamage
     }
 
     private fun triggerDefeatUniques(ourUnit: MapUnitCombatant, enemy: ICombatant, attackedTile: Tile){
@@ -340,7 +350,17 @@ object Battle {
         return true
     }
 
-    private data class DamageDealt(val attackerDealt: Int, val defenderDealt: Int)
+    /** Holder for battle result - actual damage.
+     *  @param attackerDealt Damage done by attacker to defender
+     *  @param defenderDealt Damage done by defender to attacker
+     */
+    data class DamageDealt(val attackerDealt: Int, val defenderDealt: Int) {
+        operator fun plus(other: DamageDealt) =
+            DamageDealt(attackerDealt + other.attackerDealt, defenderDealt + other.defenderDealt)
+        companion object {
+            val None = DamageDealt(0, 0)
+        }
+    }
 
     private fun takeDamage(attacker: ICombatant, defender: ICombatant): DamageDealt {
         var potentialDamageToDefender = BattleDamage.calculateDamageToDefender(attacker, defender)
@@ -358,7 +378,7 @@ object Battle {
             //so...for each round, we randomize who gets the attack in. Seems to be a good way to work for now.
 
             while (potentialDamageToDefender + potentialDamageToAttacker > 0) {
-                if (Random().nextInt(potentialDamageToDefender + potentialDamageToAttacker) < potentialDamageToDefender) {
+                if (Random.Default.nextInt(potentialDamageToDefender + potentialDamageToAttacker) < potentialDamageToDefender) {
                     potentialDamageToDefender--
                     defender.takeDamage(1)
                     if (defender.isDefeated()) break
@@ -468,7 +488,7 @@ object Battle {
     private fun postBattleMoveToAttackedTile(attacker: ICombatant, defender: ICombatant, attackedTile: Tile) {
         if (!attacker.isMelee()) return
         if (!defender.isDefeated() && defender.getCivInfo() != attacker.getCivInfo()) return
-        if (attacker is MapUnitCombatant && attacker.hasUnique(UniqueType.CannotMove)) return
+        if (attacker is MapUnitCombatant && attacker.unit.cache.cannotMove) return
 
         // This is so that if we attack e.g. a barbarian in enemy territory that we can't enter, we won't enter it
         if ((attacker as MapUnitCombatant).unit.movement.canMoveTo(attackedTile)) {
@@ -623,8 +643,8 @@ object Battle {
      * @throws IllegalArgumentException if the [attacker] and [defender] belong to the same civ.
      */
     fun captureCivilianUnit(attacker: ICombatant, defender: MapUnitCombatant, checkDefeat: Boolean = true) {
-        if (attacker.getCivInfo() == defender.getCivInfo()) {
-            throw IllegalArgumentException("Can't capture our own unit!")
+        require(attacker.getCivInfo() != defender.getCivInfo()) {
+            "Can't capture our own unit!"
         }
 
         // need to save this because if the unit is captured its owner wil be overwritten
@@ -707,7 +727,8 @@ object Battle {
             capturedUnit.civ = capturingCiv
 
             val workerTypeUnit = capturingCiv.gameInfo.ruleset.units.values
-                .firstOrNull { it.isCivilian() && it.getMatchingUniques(UniqueType.BuildImprovements).any { it.params[0] == "Land" } }
+                .firstOrNull { it.isCivilian() && it.getMatchingUniques(UniqueType.BuildImprovements)
+                    .any { unique -> unique.params[0] == "Land" } }
 
             if (workerTypeUnit != null)
                 capturingCiv.units.placeUnitNearTile(capturedUnit.currentTile.position, workerTypeUnit.name)
@@ -724,25 +745,34 @@ object Battle {
         }
     }
 
+    /**
+     *  Checks whether [nuke] is allowed to nuke [targetTile]
+     *  - Not if we would need to declare war on someone we can't.
+     *  - Disallow nuking the tile the nuke is in, as per Civ5 (but not nuking your own tiles/units otherwise)
+     *
+     *  Both [BattleTable.simulateNuke] and [SpecificUnitAutomation.automateNukes] check range, so that check is omitted here.
+     */
     fun mayUseNuke(nuke: MapUnitCombatant, targetTile: Tile): Boolean {
-        val blastRadius =
-            if (!nuke.hasUnique(UniqueType.BlastRadius)) 2
-            // Don't check conditionals as these are not supported
-            else nuke.unit.getMatchingUniques(UniqueType.BlastRadius).first().params[0].toInt()
+        if (nuke.getTile() == targetTile) return false
 
         var canNuke = true
         val attackerCiv = nuke.getCivInfo()
-        for (tile in targetTile.getTilesInDistance(blastRadius)) {
-            val defendingTileCiv = tile.getCity()?.civ
-            if (defendingTileCiv != null && attackerCiv.knows(defendingTileCiv)) {
-                canNuke = canNuke && attackerCiv.getDiplomacyManager(defendingTileCiv).canAttack()
-            }
+        fun checkDefenderCiv(defenderCiv: Civilization?) {
+            if (defenderCiv == null) return
+            // Allow nuking yourself! (Civ5 source: CvUnit::isNukeVictim)
+            if (defenderCiv == attackerCiv || defenderCiv.isDefeated()) return
+            // Gleaned from Civ5 source - this disallows nuking unknown civs even in invisible tiles
+            // https://github.com/Gedemon/Civ5-DLL/blob/master/CvGameCoreDLL_Expansion1/CvUnit.cpp#L5056
+            // https://github.com/Gedemon/Civ5-DLL/blob/master/CvGameCoreDLL_Expansion1/CvTeam.cpp#L986
+            if (attackerCiv.knows(defenderCiv) && attackerCiv.getDiplomacyManager(defenderCiv).canAttack())
+                return
+            canNuke = false
+        }
 
-            val defender = getMapCombatantOfTile(tile) ?: continue
-            val defendingUnitCiv = defender.getCivInfo()
-            if (attackerCiv.knows(defendingUnitCiv)) {
-                canNuke = canNuke && attackerCiv.getDiplomacyManager(defendingUnitCiv).canAttack()
-            }
+        val blastRadius = nuke.unit.getNukeBlastRadius()
+        for (tile in targetTile.getTilesInDistance(blastRadius)) {
+            checkDefenderCiv(tile.getOwner())
+            checkDefenderCiv(getMapCombatantOfTile(tile)?.getCivInfo())
         }
         return canNuke
     }
@@ -760,7 +790,7 @@ object Battle {
             }
         }
 
-        val strength = attacker.unit.getMatchingUniques(UniqueType.NuclearWeapon)
+        val nukeStrength = attacker.unit.getMatchingUniques(UniqueType.NuclearWeapon)
             .firstOrNull()?.params?.get(0)?.toInt() ?: return
 
         val blastRadius = attacker.unit.getMatchingUniques(UniqueType.BlastRadius)
@@ -776,10 +806,10 @@ object Battle {
         }
 
         // Declare war on all potentially hit units. They'll try to intercept the nuke before it drops
-        for(civWhoseUnitWasAttacked in hitTiles
+        for (civWhoseUnitWasAttacked in hitTiles
             .flatMap { it.getUnits() }
             .map { it.civ }.distinct()
-            .filter{it != attackingCiv}) {
+            .filter { it != attackingCiv }) {
                 tryDeclareWar(civWhoseUnitWasAttacked)
                 if (attacker.unit.baseUnit.isAirUnit() && !attacker.isDefeated()) {
                     tryInterceptAirAttack(attacker, targetTile, civWhoseUnitWasAttacked, null)
@@ -789,17 +819,9 @@ object Battle {
 
         attacker.unit.attacksSinceTurnStart.add(Vector2(targetTile.position))
 
-        // Destroy units on the target tile
-        // Needs the toList() because if we're destroying the units, they're no longer part of the sequence
-        for (defender in targetTile.getUnits().filter { it != attacker.unit }.toList()) {
-            defender.destroy()
-            postBattleNotifications(attacker, MapUnitCombatant(defender), defender.getTile())
-            destroyIfDefeated(defender.civ, attacker.getCivInfo())
-        }
-
         for (tile in hitTiles) {
             // Handle complicated effects
-            doNukeExplosionForTile(attacker, tile, strength)
+            doNukeExplosionForTile(attacker, tile, nukeStrength, targetTile == tile)
         }
 
         // Instead of postBattleAction() just destroy the unit, all other functions are not relevant
@@ -816,7 +838,12 @@ object Battle {
         }
     }
 
-    private fun doNukeExplosionForTile(attacker: MapUnitCombatant, tile: Tile, nukeStrength: Int) {
+    private fun doNukeExplosionForTile(
+        attacker: MapUnitCombatant,
+        tile: Tile,
+        nukeStrength: Int,
+        isGroundZero: Boolean
+    ) {
         // https://forums.civfanatics.com/resources/unit-guide-modern-future-units-g-k.25628/
         // https://www.carlsguides.com/strategy/civilization5/units/aircraft-nukes.ph
         // Testing done by Ravignir
@@ -827,11 +854,15 @@ object Battle {
         for (resource in attacker.unit.baseUnit.getResourceRequirementsPerTurn().keys) {
             if (civResources[resource]!! < 0 && !attacker.getCivInfo().isBarbarian())
                 damageModifierFromMissingResource *= 0.5f // I could not find a source for this number, but this felt about right
+                // - Original Civ5 does *not* reduce damage from missing resource, from source inspection
         }
 
+        var buildingModifier = 1f  // Strange, but in Civ5 a bunker mitigates damage to garrison, even if the city is destroyed by the nuke
+
         // Damage city and reduce its population
         val city = tile.getCity()
         if (city != null && tile.position == city.location) {
+            buildingModifier = city.getAggregateModifier(UniqueType.GarrisonDamageFromNukes)
             doNukeExplosionDamageToCity(city, nukeStrength, damageModifierFromMissingResource)
             postBattleNotifications(attacker, CityCombatant(city), city.getCenterTile())
             destroyIfDefeated(city.civ, attacker.getCivInfo())
@@ -839,65 +870,89 @@ object Battle {
 
         // Damage and/or destroy units on the tile
         for (unit in tile.getUnits().toList()) { // toList so if it's destroyed there's no concurrent modification
+            val damage = (when {
+                    isGroundZero || nukeStrength >= 2 -> 100
+                    // The following constants are NUKE_UNIT_DAMAGE_BASE / NUKE_UNIT_DAMAGE_RAND_1 / NUKE_UNIT_DAMAGE_RAND_2 in Civ5
+                    nukeStrength == 1 -> 30 + Random.Default.nextInt(40) + Random.Default.nextInt(40)
+                    // Level 0 does not exist in Civ5 (it treats units same as level 2)
+                    else -> 20 + Random.Default.nextInt(30)
+                } * buildingModifier * damageModifierFromMissingResource + 1f.ulp).toInt()
             val defender = MapUnitCombatant(unit)
-            if (defender.unit.isCivilian() || nukeStrength >= 2) {
-                unit.destroy()
-            } else if (nukeStrength == 1) {
-                defender.takeDamage(((40 + Random().nextInt(60)) * damageModifierFromMissingResource).toInt())
-            } else if (nukeStrength == 0) {
-                defender.takeDamage(((20 + Random().nextInt(30)) * damageModifierFromMissingResource).toInt())
+            if (unit.isCivilian()) {
+                if (unit.health - damage <= 40) unit.destroy()  // Civ5: NUKE_NON_COMBAT_DEATH_THRESHOLD = 60
+            } else {
+                defender.takeDamage(damage)
             }
             postBattleNotifications(attacker, defender, defender.getTile())
             destroyIfDefeated(defender.getCivInfo(), attacker.getCivInfo())
         }
 
         // Pillage improvements, pillage roads, add fallout
-        if (tile.getUnpillagedImprovement() != null && !tile.getTileImprovement()!!.hasUnique(UniqueType.Irremovable)) {
-            if (tile.getTileImprovement()!!.hasUnique(UniqueType.Unpillagable)) {
-                tile.changeImprovement(null)
-            } else {
-                tile.setPillaged()
-            }
-        }
-        if (tile.getUnpillagedRoad() != RoadStatus.None)
-            tile.setPillaged()
-        if (tile.isLand && !tile.isImpassible() && !tile.isCityCenter()) {
-            if (tile.terrainHasUnique(UniqueType.DestroyableByNukesChance)) {
-                for (terrainFeature in tile.terrainFeatureObjects) {
-                    for (unique in terrainFeature.getMatchingUniques(UniqueType.DestroyableByNukesChance)) {
-                        if (Random().nextFloat() >= unique.params[0].toFloat() / 100f) continue
-                        tile.removeTerrainFeature(terrainFeature.name)
-                        if (!tile.terrainFeatures.contains("Fallout"))
-                            tile.addTerrainFeature("Fallout")
-                    }
+        if (tile.isCityCenter()) return  // Never touch city centers - if they survived
+        fun applyPillageAndFallout() {
+            if (tile.getUnpillagedImprovement() != null && !tile.getTileImprovement()!!.hasUnique(UniqueType.Irremovable)) {
+                if (tile.getTileImprovement()!!.hasUnique(UniqueType.Unpillagable)) {
+                    tile.changeImprovement(null)
+                } else {
+                    tile.setPillaged()
                 }
-            } else if (Random().nextFloat() < 0.5f && !tile.terrainFeatures.contains("Fallout")) {
-                tile.addTerrainFeature("Fallout")
             }
+            if (tile.getUnpillagedRoad() != RoadStatus.None)
+                tile.setPillaged()
+            if (tile.isWater || tile.isImpassible() || tile.terrainFeatures.contains("Fallout")) return
+            tile.addTerrainFeature("Fallout")
+        }
+
+        if (tile.terrainHasUnique(UniqueType.DestroyableByNukesChance)) {
+            // Note: Safe from concurrent modification exceptions only because removeTerrainFeature
+            // *replaces* terrainFeatureObjects and the loop will continue on the old one
+            for (terrainFeature in tile.terrainFeatureObjects) {
+                for (unique in terrainFeature.getMatchingUniques(UniqueType.DestroyableByNukesChance)) {
+                    val chance = unique.params[0].toFloat() / 100f
+                    if (!(chance > 0f && isGroundZero) && Random.Default.nextFloat() >= chance) continue
+                    tile.removeTerrainFeature(terrainFeature.name)
+                    applyPillageAndFallout()
+                }
+            }
+        } else if (isGroundZero || Random.Default.nextFloat() < 0.5f) {  // Civ5: NUKE_FALLOUT_PROB
+            applyPillageAndFallout()
         }
     }
 
+    /** @return the "protection" modifier from buildings (Bomb Shelter, UniqueType.PopulationLossFromNukes) */
     private fun doNukeExplosionDamageToCity(targetedCity: City, nukeStrength: Int, damageModifierFromMissingResource: Float) {
-        if (nukeStrength > 1 && targetedCity.population.population < 5 && targetedCity.canBeDestroyed(true)) {
+        // Original Capitals must be protected, `canBeDestroyed` is responsible for that check.
+        // The `justCaptured = true` parameter is what allows other Capitals to suffer normally.
+        if ((nukeStrength > 2 || nukeStrength > 1 && targetedCity.population.population < 5)
+                && targetedCity.canBeDestroyed(true)) {
             targetedCity.destroyCity()
             return
         }
+
         val cityCombatant = CityCombatant(targetedCity)
         cityCombatant.takeDamage((cityCombatant.getHealth() * 0.5f * damageModifierFromMissingResource).toInt())
 
-        var populationLoss = targetedCity.population.population *
-            when (nukeStrength) {
-                0 -> 0f
-                1 -> (30 + Random().nextInt(40)) / 100f
-                2 -> (60 + Random().nextInt(20)) / 100f
-                else -> 1f
-            }
-        for (unique in targetedCity.getMatchingUniques(UniqueType.PopulationLossFromNukes)) {
-            if (!targetedCity.matchesFilter(unique.params[1])) continue
-            populationLoss *= unique.params[0].toPercent()
+        // Difference to original: Civ5 rounds population loss down twice - before and after bomb shelters
+        val populationLoss = (
+                targetedCity.population.population *
+                    targetedCity.getAggregateModifier(UniqueType.PopulationLossFromNukes) *
+                when (nukeStrength) {
+                    0 -> 0f
+                    1 -> (30 + Random.Default.nextInt(20) + Random.Default.nextInt(20)) / 100f
+                    2 -> (60 + Random.Default.nextInt(10) + Random.Default.nextInt(10)) / 100f
+                    else -> 1f  // hypothetical nukeStrength 3 -> always to 1 pop
+                }
+            ).toInt().coerceAtMost(targetedCity.population.population - 1)
+        targetedCity.population.addPopulation(-populationLoss)
+    }
+
+    private fun City.getAggregateModifier(uniqueType: UniqueType): Float {
+        var modifier = 1f
+        for (unique in getMatchingUniques(uniqueType)) {
+            if (!matchesFilter(unique.params[1])) continue
+            modifier *= unique.params[0].toPercent()
         }
-        targetedCity.population.addPopulation(-populationLoss.toInt())
-        if (targetedCity.population.population < 1) targetedCity.population.setPopulation(1)
+        return modifier
     }
 
     // Should draw an Interception if available on the tile from any Civ
@@ -1013,62 +1068,70 @@ object Battle {
         attacker.unit.action = null
     }
 
-    private fun tryInterceptAirAttack(attacker: MapUnitCombatant, attackedTile: Tile, interceptingCiv: Civilization, defender: ICombatant?) {
+    private fun tryInterceptAirAttack(
+        attacker: MapUnitCombatant,
+        attackedTile: Tile,
+        interceptingCiv: Civilization,
+        defender: ICombatant?
+    ): DamageDealt {
         if (attacker.unit.hasUnique(UniqueType.CannotBeIntercepted, StateForConditionals(attacker.getCivInfo(), ourCombatant = attacker, theirCombatant = defender, attackedTile = attackedTile)))
-            return
+            return DamageDealt.None
+
         // Pick highest chance interceptor
-        for (interceptor in interceptingCiv.units.getCivUnits()
+        val interceptor = interceptingCiv.units.getCivUnits()
             .filter { it.canIntercept(attackedTile) }
             .sortedByDescending { it.interceptChance() }
-        ) {
-            // Can't intercept if we have a unique preventing it
-            val conditionalState = StateForConditionals(interceptingCiv, ourCombatant = MapUnitCombatant(interceptor), theirCombatant = attacker, combatAction = CombatAction.Intercept, attackedTile = attackedTile)
-            if (interceptor.getMatchingUniques(UniqueType.CannotInterceptUnits, conditionalState)
-                .any { attacker.matchesCategory(it.params[0]) }
-            ) continue
+            .firstOrNull { unit ->
+                // Can't intercept if we have a unique preventing it
+                val conditionalState = StateForConditionals(interceptingCiv, ourCombatant = MapUnitCombatant(unit), theirCombatant = attacker, combatAction = CombatAction.Intercept, attackedTile = attackedTile)
+                unit.getMatchingUniques(UniqueType.CannotInterceptUnits, conditionalState)
+                    .none { attacker.matchesCategory(it.params[0]) }
+                // Defender can't intercept either
+                && unit != (defender as? MapUnitCombatant)?.unit
+            }
+            ?: return DamageDealt.None
 
-            // Defender can't intercept either
-            if (defender != null && defender is MapUnitCombatant && interceptor == defender.unit) continue
-            interceptor.attacksThisTurn++  // even if you miss, you took the shot
-            // Does Intercept happen? If not, exit
-            if (Random().nextFloat() > interceptor.interceptChance() / 100f) return
+        interceptor.attacksThisTurn++  // even if you miss, you took the shot
+        // Does Intercept happen? If not, exit
+        if (Random.Default.nextFloat() > interceptor.interceptChance() / 100f)
+            return DamageDealt.None
 
-            var damage = BattleDamage.calculateDamageToDefender(
-                MapUnitCombatant(interceptor),
-                attacker
-            )
+        var damage = BattleDamage.calculateDamageToDefender(
+            MapUnitCombatant(interceptor),
+            attacker
+        )
 
-            var damageFactor = 1f + interceptor.interceptDamagePercentBonus().toFloat() / 100f
-            damageFactor *= attacker.unit.receivedInterceptDamageFactor()
+        var damageFactor = 1f + interceptor.interceptDamagePercentBonus().toFloat() / 100f
+        damageFactor *= attacker.unit.receivedInterceptDamageFactor()
 
-            damage = (damage.toFloat() * damageFactor).toInt()
+        damage = (damage.toFloat() * damageFactor).toInt().coerceAtMost(attacker.unit.health)
 
-            attacker.takeDamage(damage)
-            if (damage > 0)
-                addXp(MapUnitCombatant(interceptor), 2, attacker)
+        attacker.takeDamage(damage)
+        if (damage > 0)
+            addXp(MapUnitCombatant(interceptor), 2, attacker)
 
-            val attackerName = attacker.getName()
-            val interceptorName = interceptor.name
-            val locations = LocationAction(interceptor.currentTile.position, attacker.unit.currentTile.position)
+        val attackerName = attacker.getName()
+        val interceptorName = interceptor.name
+        val locations = LocationAction(interceptor.currentTile.position, attacker.unit.currentTile.position)
 
-            val attackerText = if (!attacker.isDefeated())
-                "Our [$attackerName] ([-$damage] HP) was attacked by an intercepting [$interceptorName] ([-0] HP)"
-            else if (interceptor.getTile() in attacker.getCivInfo().viewableTiles)
-                "Our [$attackerName] ([-$damage] HP) was destroyed by an intercepting [$interceptorName] ([-0] HP)"
-            else "Our [$attackerName] ([-$damage] HP) was destroyed by an unknown interceptor"
+        val attackerText = if (!attacker.isDefeated())
+            "Our [$attackerName] ([-$damage] HP) was attacked by an intercepting [$interceptorName] ([-0] HP)"
+        else if (interceptor.getTile() in attacker.getCivInfo().viewableTiles)
+            "Our [$attackerName] ([-$damage] HP) was destroyed by an intercepting [$interceptorName] ([-0] HP)"
+        else "Our [$attackerName] ([-$damage] HP) was destroyed by an unknown interceptor"
 
-            attacker.getCivInfo().addNotification(
-                attackerText, interceptor.currentTile.position, NotificationCategory.War,
-                attackerName, NotificationIcon.War, interceptorName
-            )
+        attacker.getCivInfo().addNotification(
+            attackerText, interceptor.currentTile.position, NotificationCategory.War,
+            attackerName, NotificationIcon.War, interceptorName
+        )
 
-            val interceptorText = if (attacker.isDefeated())
-                "Our [$interceptorName] ([-0] HP) intercepted and destroyed an enemy [$attackerName] ([-$damage] HP)"
-            else "Our [$interceptorName] ([-0] HP) intercepted and attacked an enemy [$attackerName] ([-$damage] HP)"
-            interceptingCiv.addNotification(interceptorText, locations, NotificationCategory.War,
-                    interceptorName, NotificationIcon.War, attackerName)
-            return
-        }
+        val interceptorText = if (attacker.isDefeated())
+            "Our [$interceptorName] ([-0] HP) intercepted and destroyed an enemy [$attackerName] ([-$damage] HP)"
+        else "Our [$interceptorName] ([-0] HP) intercepted and attacked an enemy [$attackerName] ([-$damage] HP)"
+        interceptingCiv.addNotification(interceptorText, locations, NotificationCategory.War,
+                interceptorName, NotificationIcon.War, attackerName)
+
+        return DamageDealt(0, damage)
     }
 
     private fun doWithdrawFromMeleeAbility(attacker: ICombatant, defender: ICombatant, baseWithdrawChance: Int): Boolean {
@@ -1081,7 +1144,7 @@ object Battle {
         if (attacker !is MapUnitCombatant) return false         // allow simple access to unit property
         if (defender !is MapUnitCombatant) return false
         if (defender.unit.isEmbarked()) return false
-        if (defender.hasUnique(UniqueType.CannotMove)) return false
+        if (defender.unit.cache.cannotMove) return false
         // Promotions have no effect as per what I could find in available documentation
         val attackBaseUnit = attacker.unit.baseUnit
         val defendBaseUnit = defender.unit.baseUnit
@@ -1138,7 +1201,13 @@ object Battle {
         if (!attackedTile.getTileImprovement()!!.hasUnique(UniqueType.Unpillagable)
             && attacker.hasUnique(UniqueType.DestroysImprovementUponAttack, conditionalState)
         ) {
+            val currentTileImprovement = attackedTile.improvement
             attackedTile.changeImprovement(null)
+            defender.getCivInfo().addNotification(
+                "An enemy [${attacker.unit.baseUnit.name}] has destroyed our tile improvement [${currentTileImprovement}]",
+                LocationAction(attackedTile.position, attacker.getTile().position),
+                NotificationCategory.War, attacker.unit.baseUnit.name,
+                NotificationIcon.War)
         }
     }
 }
