@@ -7,7 +7,9 @@ import com.unciv.logic.map.BFS
 import com.unciv.logic.map.HexMath.getDistance
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
+import com.unciv.models.UnitActionType
 import com.unciv.models.helpers.UnitMovementMemoryType
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 
 class UnitMovement(val unit: MapUnit) {
@@ -17,7 +19,7 @@ class UnitMovement(val unit: MapUnit) {
     fun getEnemyMovementPenalty(civInfo:Civilization, enemyUnit: MapUnit): Float {
         if (civInfo.enemyMovementPenaltyUniques != null && civInfo.enemyMovementPenaltyUniques!!.any()) {
             return civInfo.enemyMovementPenaltyUniques!!.sumOf {
-                if (it.type!! == UniqueType.EnemyLandUnitsSpendExtraMovement
+                if (it.type!! == UniqueType.EnemyUnitsSpendExtraMovement
                         && enemyUnit.matchesFilter(it.params[0]))
                     it.params[1].toInt()
                 else 0
@@ -33,8 +35,9 @@ class UnitMovement(val unit: MapUnit) {
         civInfo: Civilization,
         considerZoneOfControl: Boolean = true
     ): Float {
+        if (unit.cache.cannotMove) return 100f
 
-        if (from.isLand != to.isLand && unit.baseUnit.isLandUnit())
+        if (from.isLand != to.isLand && unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater)
             return if (from.isWater && to.isLand) unit.cache.costToDisembark ?: 100f
             else unit.cache.costToEmbark ?: 100f
 
@@ -60,7 +63,8 @@ class UnitMovement(val unit: MapUnit) {
         // when entering territory of a city state
         val areConnectedByRoad = from.hasConnection(civInfo) && to.hasConnection(civInfo)
 
-        val areConnectedByRiver = from.isAdjacentToRiver() && to.isAdjacentToRiver() && from.isConnectedByRiver(to)
+        val areConnectedByRiver =
+            from.isAdjacentToRiver() && to.isAdjacentToRiver() && from.isConnectedByRiver(to)
 
         if (areConnectedByRoad && (!areConnectedByRiver || civInfo.tech.roadsConnectAcrossRivers))
             return unit.civ.tech.movementSpeedOnRoads + extraCost
@@ -73,7 +77,29 @@ class UnitMovement(val unit: MapUnit) {
         if (unit.cache.noTerrainMovementUniques)
             return terrainCost + extraCost
 
-        if (to.terrainFeatures.any { unit.cache.doubleMovementInTerrain[it] == MapUnitCache.DoubleMovementTerrainTarget.Feature })
+        val stateForConditionals = StateForConditionals(unit.civ, unit = unit, tile = to)
+        fun matchesTerrainTarget(
+            doubleMovement: MapUnitCache.DoubleMovement,
+            target: MapUnitCache.DoubleMovementTerrainTarget
+        ): Boolean {
+            if (doubleMovement.terrainTarget != target) return false
+            if (doubleMovement.unique.conditionals.isNotEmpty()) {
+                if (!doubleMovement.unique.conditionalsApply(stateForConditionals)) return false
+            }
+
+            return true
+        }
+
+        fun matchesTerrainTarget(
+            terrainName: String,
+            target: MapUnitCache.DoubleMovementTerrainTarget
+        ): Boolean {
+            val doubleMovement = unit.cache.doubleMovementInTerrain[terrainName] ?: return false
+            return matchesTerrainTarget(doubleMovement, target)
+        }
+
+
+        if (to.terrainFeatures.any { matchesTerrainTarget(it, MapUnitCache.DoubleMovementTerrainTarget.Feature) })
             return terrainCost * 0.5f + extraCost
 
         if (unit.cache.roughTerrainPenalty && to.isRoughTerrain())
@@ -86,17 +112,18 @@ class UnitMovement(val unit: MapUnit) {
         if (unit.cache.noBaseTerrainOrHillDoubleMovementUniques)
             return terrainCost + extraCost
 
-        if (unit.cache.doubleMovementInTerrain[to.baseTerrain] == MapUnitCache.DoubleMovementTerrainTarget.Base)
+        if (matchesTerrainTarget(to.baseTerrain, MapUnitCache.DoubleMovementTerrainTarget.Base))
             return terrainCost * 0.5f + extraCost
-        if (unit.cache.doubleMovementInTerrain[Constants.hill] == MapUnitCache.DoubleMovementTerrainTarget.Hill && to.isHill())
+        if (matchesTerrainTarget(Constants.hill, MapUnitCache.DoubleMovementTerrainTarget.Hill)
+            && to.isHill())
             return terrainCost * 0.5f + extraCost
 
         if (unit.cache.noFilteredDoubleMovementUniques)
             return terrainCost + extraCost
         if (unit.cache.doubleMovementInTerrain.any {
-            it.value == MapUnitCache.DoubleMovementTerrainTarget.Filter &&
-                to.matchesFilter(it.key)
-        })
+                matchesTerrainTarget(it.value, MapUnitCache.DoubleMovementTerrainTarget.Filter)
+                    && to.matchesFilter(it.key)
+            })
             return terrainCost * 0.5f + extraCost
 
         return terrainCost + extraCost // no road or other movement cost reduction
@@ -153,14 +180,24 @@ class UnitMovement(val unit: MapUnit) {
      * Does not consider if tiles can actually be entered, use canMoveTo for that.
      * If a tile can be reached within the turn, but it cannot be passed through, the total distance to it is set to unitMovement
      */
-    fun getDistanceToTilesWithinTurn(origin: Vector2, unitMovement: Float, considerZoneOfControl: Boolean = true, tilesToIgnore: HashSet<Tile>? = null): PathsToTilesWithinTurn {
+    fun getDistanceToTilesWithinTurn(
+                origin: Vector2,
+                unitMovement: Float,
+                considerZoneOfControl: Boolean = true,
+                tilesToIgnore: HashSet<Tile>? = null,
+                passThroughCache: HashMap<Tile, Boolean> = HashMap(),
+                movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap()
+            ): PathsToTilesWithinTurn {
         val distanceToTiles = PathsToTilesWithinTurn()
-        if (unitMovement == 0f) return distanceToTiles
 
         val currentUnitTile = unit.currentTile
         // This is for performance, because this is called all the time
         val unitTile = if (origin == currentUnitTile.position) currentUnitTile else currentUnitTile.tileMap[origin]
         distanceToTiles[unitTile] = ParentTileAndTotalDistance(unitTile, unitTile, 0f)
+
+        // If I can't move my only option is to stay...
+        if (unitMovement == 0f || unit.cache.cannotMove) return distanceToTiles
+
         var tilesToCheck = listOf(unitTile)
 
         while (tilesToCheck.isNotEmpty()) {
@@ -169,15 +206,19 @@ class UnitMovement(val unit: MapUnit) {
                 for (neighbor in tileToCheck.neighbors) {
                     if (tilesToIgnore?.contains(neighbor) == true) continue // ignore this tile
                     var totalDistanceToTile: Float = when {
-                        !unit.civ.hasExplored(neighbor) ->
+                        !neighbor.isExplored(unit.civ) ->
                             distanceToTiles[tileToCheck]!!.totalDistance + 1f  // If we don't know then we just guess it to be 1.
-                        !canPassThrough(neighbor) -> unitMovement // Can't go here.
+                        !passThroughCache.getOrPut(neighbor) { canPassThrough(neighbor) } -> unitMovement // Can't go here.
                         // The reason that we don't just "return" is so that when calculating how to reach an enemy,
                         // You need to assume his tile is reachable, otherwise all movement algorithms on reaching enemy
                         // cities and units goes kaput.
                         else -> {
-                            val distanceBetweenTiles = getMovementCostBetweenAdjacentTiles(tileToCheck, neighbor, unit.civ, considerZoneOfControl)
-                            distanceToTiles[tileToCheck]!!.totalDistance + distanceBetweenTiles
+                            val key = Pair(tileToCheck, neighbor)
+                            val movementCost =
+                                    movementCostCache.getOrPut(key) {
+                                        getMovementCostBetweenAdjacentTiles(tileToCheck, neighbor, unit.civ, considerZoneOfControl)
+                                    }
+                            distanceToTiles[tileToCheck]!!.totalDistance + movementCost
                         }
                     }
 
@@ -204,7 +245,8 @@ class UnitMovement(val unit: MapUnit) {
      * Returns an empty list if there's no way to get to the destination.
      */
     fun getShortestPath(destination: Tile, avoidDamagingTerrain: Boolean = false): List<Tile> {
-        if (unit.hasUnique(UniqueType.CannotMove)) return listOf()
+        if (unit.cache.cannotMove) return listOf()
+
         // First try and find a path without damaging terrain
         if (!avoidDamagingTerrain && unit.civ.passThroughImpassableUnlocked && unit.baseUnit.isLandUnit()) {
             val damageFreePath = getShortestPath(destination, true)
@@ -232,18 +274,17 @@ class UnitMovement(val unit: MapUnit) {
         val movementTreeParents = HashMap<Tile, Tile?>() // contains a map of "you can get from X to Y in that turn"
         movementTreeParents[currentTile] = null
 
-        var movementThisTurn = unit.currentMovement
         var distance = 1
+        val unitMaxMovement = unit.getMaxMovement().toFloat()
         val newTilesToCheck = ArrayList<Tile>()
-        var considerZoneOfControl = true // only for first distance!
         val visitedTiles: HashSet<Tile> = hashSetOf(currentTile)
         val civilization = unit.civ
 
+        val passThroughCache = HashMap<Tile, Boolean>()
+        val movementCostCache = HashMap<Pair<Tile, Tile>, Float>()
+        val canMoveToCache = HashMap<Tile, Boolean>()
+
         while (true) {
-            if (distance == 2) { // only set this once after distance > 1
-                movementThisTurn = unit.getMaxMovement().toFloat()
-                considerZoneOfControl = false  // by then units would have moved around, we don't need to consider untenable futures when it harms performance!
-            }
             newTilesToCheck.clear()
 
             var tilesByPreference = tilesToCheck.sortedBy { it.aerialDistanceTo(destination) }
@@ -253,10 +294,17 @@ class UnitMovement(val unit: MapUnit) {
 
             for (tileToCheck in tilesByPreference) {
                 val distanceToTilesThisTurn = if (distance == 1) {
-                    getDistanceToTiles(considerZoneOfControl) // check cache
+                    getDistanceToTiles(true, passThroughCache, movementCostCache) // check cache
                 }
                 else {
-                    getDistanceToTilesWithinTurn(tileToCheck.position, movementThisTurn, considerZoneOfControl, visitedTiles)
+                    getDistanceToTilesWithinTurn(
+                        tileToCheck.position,
+                        unitMaxMovement,
+                        false,
+                        visitedTiles,
+                        passThroughCache,
+                        movementCostCache
+                    )
                 }
                 for (reachableTile in distanceToTilesThisTurn.keys) {
                     // Avoid damaging terrain on first pass
@@ -280,7 +328,9 @@ class UnitMovement(val unit: MapUnit) {
                     } else {
                         if (movementTreeParents.containsKey(reachableTile)) continue // We cannot be faster than anything existing...
                         if (!isUnknownTileWeShouldAssumeToBePassable(reachableTile) &&
-                                !canMoveTo(reachableTile)) continue // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
+                                !canMoveToCache.getOrPut(reachableTile) { canMoveTo(reachableTile) })
+                            // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
+                            continue
                         movementTreeParents[reachableTile] = tileToCheck
                         newTilesToCheck.add(reachableTile)
                     }
@@ -348,14 +398,14 @@ class UnitMovement(val unit: MapUnit) {
     /** This is performance-heavy - use as last resort, only after checking everything else!
      * Also note that REACHABLE tiles are not necessarily tiles that the unit CAN ENTER */
     fun canReach(destination: Tile): Boolean {
-        if (unit.hasUnique(UniqueType.CannotMove)) return false
+        if (unit.cache.cannotMove) return destination == unit.getTile()
         if (unit.baseUnit.movesLikeAirUnits() || unit.isPreparingParadrop())
             return canReachInCurrentTurn(destination)
         return getShortestPath(destination).any()
     }
 
     private fun canReachInCurrentTurn(destination: Tile): Boolean {
-        if (unit.hasUnique(UniqueType.CannotMove)) return false
+        if (unit.cache.cannotMove) return destination == unit.getTile()
         if (unit.baseUnit.movesLikeAirUnits())
             return unit.currentTile.aerialDistanceTo(destination) <= unit.getMaxMovementForAirUnits()
         if (unit.isPreparingParadrop())
@@ -365,7 +415,7 @@ class UnitMovement(val unit: MapUnit) {
 
     fun getReachableTilesInCurrentTurn(): Sequence<Tile> {
         return when {
-            unit.hasUnique(UniqueType.CannotMove) -> emptySequence()
+            unit.cache.cannotMove -> sequenceOf(unit.getTile())
             unit.baseUnit.movesLikeAirUnits() ->
                 unit.getTile().getTilesInDistanceRange(IntRange(1, unit.getMaxMovementForAirUnits()))
             unit.isPreparingParadrop() ->
@@ -395,7 +445,7 @@ class UnitMovement(val unit: MapUnit) {
         if (unit.baseUnit.movesLikeAirUnits()) return false
         // We can't swap with ourself
         if (reachableTile == unit.getTile()) return false
-        if (unit.hasUnique(UniqueType.CannotMove)) return false
+        if (unit.cache.cannotMove) return false
         // Check whether the tile contains a unit of the same type as us that we own and that can
         // also reach our tile in its current turn.
         val otherUnit = (
@@ -406,7 +456,7 @@ class UnitMovement(val unit: MapUnit) {
         ) ?: return false
         val ourPosition = unit.getTile()
         if (otherUnit.owner != unit.owner
-                || otherUnit.hasUnique(UniqueType.CannotMove)
+                || otherUnit.cache.cannotMove  // redundant, line below would cover it too
                 || !otherUnit.movement.canReachInCurrentTurn(ourPosition)) return false
         // Check if we could enter their tile if they wouldn't be there
         otherUnit.removeFromTile()
@@ -435,6 +485,8 @@ class UnitMovement(val unit: MapUnit) {
         var allowedTile: Tile? = null
         var distance = 0
         // When we didn't limit the allowed distance the game would sometimes spend a whole minute looking for a suitable tile.
+
+        if (canPassThrough(unit.getTile())) return // This unit can stay here - e.g. it has "May enter foreign tiles without open borders"
         while (allowedTile == null && distance < 5) {
             distance++
             allowedTile = unit.getTile().getTilesAtDistance(distance)
@@ -477,7 +529,7 @@ class UnitMovement(val unit: MapUnit) {
 
 
         if (unit.baseUnit.movesLikeAirUnits()) { // air units move differently from all other units
-            unit.action = null
+            if (unit.action != UnitActionType.Automate.value) unit.action = null
             unit.removeFromTile()
             unit.isTransported = false // it has left the carrier by own means
             unit.putInTile(destination)
@@ -682,7 +734,7 @@ class UnitMovement(val unit: MapUnit) {
 
     // Can a paratrooper land at this tile?
     fun canParadropOn(destination: Tile): Boolean {
-        if (unit.hasUnique(UniqueType.CannotMove)) return false
+        if (unit.cache.cannotMove) return false
         // Can only move to land tiles within range that are visible and not impassible
         // Based on some testing done in the base game
         if (!destination.isLand || destination.isImpassible() || !unit.civ.viewableTiles.contains(destination)) return false
@@ -717,7 +769,7 @@ class UnitMovement(val unit: MapUnit) {
                     unit.civ.getMatchingUniques(UniqueType.UnitsMayEnterOcean)
                         .any { unit.matchesFilter(it.params[0]) }
         }
-        if (tile.isWater && unit.baseUnit.isLandUnit()) {
+        if (tile.isWater && unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater) {
             if (!unit.civ.tech.unitsCanEmbark) return false
             if (tile.isOcean && !unit.civ.tech.embarkedUnitsCanEnterOcean && !unitSpecificAllowOcean)
                 return false
@@ -726,6 +778,8 @@ class UnitMovement(val unit: MapUnit) {
             if (!unitSpecificAllowOcean && unit.cache.cannotEnterOceanTiles) return false
         }
 
+        if (unit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence) && tile.getOwner()?.isCityState() == true)
+            return true
         if (!unit.cache.canEnterForeignTerrain && !tile.canCivPassThrough(unit.civ)) return false
 
         // The first unit is:
@@ -737,7 +791,7 @@ class UnitMovement(val unit: MapUnit) {
         if (firstUnit != null && unit.civ != firstUnit.civ) {
             // Allow movement through unguarded, at-war Civilian Unit. Capture on the way
             // But not for Embarked Units capturing on Water
-            if (!(unit.baseUnit.isLandUnit() && tile.isWater)
+            if (!(unit.baseUnit.isLandUnit() && tile.isWater && !unit.cache.canMoveOnWater)
                     && firstUnit.isCivilian() && unit.civ.isAtWarWith(firstUnit.civ))
                 return true
             // Cannot enter hostile tile with any unit in there
@@ -749,12 +803,23 @@ class UnitMovement(val unit: MapUnit) {
     }
 
 
-    fun getDistanceToTiles(considerZoneOfControl: Boolean = true): PathsToTilesWithinTurn {
+    fun getDistanceToTiles(
+                considerZoneOfControl: Boolean = true,
+                passThroughCache: HashMap<Tile, Boolean> = HashMap(),
+                movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap())
+            : PathsToTilesWithinTurn {
         val cacheResults = pathfindingCache.getDistanceToTiles(considerZoneOfControl)
         if (cacheResults != null) {
             return cacheResults
         }
-        val distanceToTiles = getDistanceToTilesWithinTurn(unit.currentTile.position, unit.currentMovement, considerZoneOfControl)
+        val distanceToTiles = getDistanceToTilesWithinTurn(
+            unit.currentTile.position,
+            unit.currentMovement,
+            considerZoneOfControl,
+            null,
+            passThroughCache,
+            movementCostCache
+        )
         pathfindingCache.setDistanceToTiles(considerZoneOfControl, distanceToTiles)
         return distanceToTiles
     }
