@@ -55,6 +55,7 @@ class ModOptions : IHasUniques {
 
     var lastUpdated = ""
     var modUrl = ""
+    var defaultBranch = "master"
     var author = ""
     var modSize = 0
     var topics = mutableListOf<String>()
@@ -90,6 +91,7 @@ class Ruleset {
     val quests = LinkedHashMap<String, Quest>()
     val specialists = LinkedHashMap<String, Specialist>()
     val technologies = LinkedHashMap<String, Technology>()
+    val techColumns = ArrayList<TechColumn>()
     val terrains = LinkedHashMap<String, Terrain>()
     val tileImprovements = LinkedHashMap<String, TileImprovement>()
     val tileResources = LinkedHashMap<String, TileResource>()
@@ -110,8 +112,10 @@ class Ruleset {
 
     private fun <T : INamed> createHashmap(items: Array<T>): LinkedHashMap<String, T> {
         val hashMap = LinkedHashMap<String, T>(items.size)
-        for (item in items)
+        for (item in items) {
             hashMap[item.name] = item
+            (item as? IRulesetObject)?.originRuleset = name
+        }
         return hashMap
     }
 
@@ -151,6 +155,7 @@ class Ruleset {
                 technologies.remove(it)
             }
         technologies.putAll(ruleset.technologies)
+        techColumns.addAll(ruleset.techColumns)
         terrains.putAll(ruleset.terrains)
         tileImprovements.putAll(ruleset.tileImprovements)
         tileResources.putAll(ruleset.tileResources)
@@ -159,7 +164,7 @@ class Ruleset {
         cityStateTypes.putAll(ruleset.cityStateTypes)
         ruleset.modOptions.unitsToRemove
             .flatMap { unitToRemove ->
-                units.filter { it.apply { value.ruleset=this@Ruleset }.value.matchesFilter(unitToRemove) }.keys
+                units.filter { it.apply { value.ruleset = this@Ruleset }.value.matchesFilter(unitToRemove) }.keys
             }.toSet().forEach {
                 units.remove(it)
             }
@@ -167,30 +172,7 @@ class Ruleset {
         modOptions.uniques.addAll(ruleset.modOptions.uniques)
         modOptions.constants.merge(ruleset.modOptions.constants)
 
-        // Allow each mod to define their own columns, and if there's a conflict, later mods will be shifted right
-        // We should never be editing the original ruleset objects, only copies
-        val addRulesetUnitPromotionClones = ruleset.unitPromotions.values.map { it.clone() }
-        val existingPromotionLocations =
-                unitPromotions.values.map { "${it.row}/${it.column}" }.toHashSet()
-        val promotionsWithConflictingLocations = addRulesetUnitPromotionClones.filter {
-            existingPromotionLocations.contains("${it.row}/${it.column}")
-        }
-        val columnsWithConflictingLocations =
-                promotionsWithConflictingLocations.map { it.column }.distinct()
-
-        if (columnsWithConflictingLocations.isNotEmpty()) {
-            var highestExistingColumn = unitPromotions.values.maxOf { it.column }
-            for (conflictingColumn in columnsWithConflictingLocations) {
-                highestExistingColumn += 1
-                val newColumn = highestExistingColumn
-                for (promotion in addRulesetUnitPromotionClones)
-                    if (promotion.column == conflictingColumn)
-                        promotion.column = newColumn
-            }
-        }
-        val finalModUnitPromotionsMap = addRulesetUnitPromotionClones.associateBy { it.name }
-
-        unitPromotions.putAll(finalModUnitPromotionsMap)
+        unitPromotions.putAll(ruleset.unitPromotions)
 
         mods += ruleset.mods
     }
@@ -211,6 +193,7 @@ class Ruleset {
         ruinRewards.clear()
         specialists.clear()
         technologies.clear()
+        techColumns.clear()
         terrains.clear()
         tileImprovements.clear()
         tileResources.clear()
@@ -247,12 +230,16 @@ class Ruleset {
             allRulesetObjects() + sequenceOf(modOptions)
 
     fun load(folderHandle: FileHandle) {
+        // Note: Most files are loaded using createHashmap, which sets originRuleset automatically.
+        // For other files containing IRulesetObject's we'll have to remember to do so manually - e.g. Tech.
         val modOptionsFile = folderHandle.child("ModOptions.json")
         if (modOptionsFile.exists()) {
             try {
                 modOptions = json().fromJsonFile(ModOptions::class.java, modOptionsFile)
                 modOptions.updateDeprecations()
-            } catch (ex: Exception) {}
+            } catch (ex: Exception) {
+                Log.error("Failed to get modOptions from json file", ex)
+            }
             modOptions.uniqueObjects = modOptions.uniques.map { Unique(it, UniqueTarget.ModOptions) }
             modOptions.uniqueMap = modOptions.uniqueObjects.groupBy { it.placeholderText }
         }
@@ -261,9 +248,11 @@ class Ruleset {
         if (techFile.exists()) {
             val techColumns = json().fromJsonFile(Array<TechColumn>::class.java, techFile)
             for (techColumn in techColumns) {
+                this.techColumns.add(techColumn)
                 for (tech in techColumn.techs) {
                     if (tech.cost == 0) tech.cost = techColumn.techCost
                     tech.column = techColumn
+                    tech.originRuleset = name
                     technologies[tech.name] = tech
                 }
             }
@@ -275,7 +264,10 @@ class Ruleset {
         val terrainsFile = folderHandle.child("Terrains.json")
         if (terrainsFile.exists()) {
             terrains += createHashmap(json().fromJsonFile(Array<Terrain>::class.java, terrainsFile))
-            for (terrain in terrains.values) terrain.setTransients()
+            for (terrain in terrains.values) {
+                terrain.originRuleset = name
+                terrain.setTransients()
+            }
         }
 
         val resourcesFile = folderHandle.child("TileResources.json")
@@ -305,14 +297,6 @@ class Ruleset {
         val promotionsFile = folderHandle.child("UnitPromotions.json")
         if (promotionsFile.exists()) unitPromotions += createHashmap(json().fromJsonFile(Array<Promotion>::class.java, promotionsFile))
 
-        var topRow = unitPromotions.values.filter { it.column == 0 }.maxOfOrNull { it.row } ?: -1
-        for (promotion in unitPromotions.values)
-            if (promotion.row == -1){
-                promotion.column = 0
-                topRow += 1
-                promotion.row = topRow
-            }
-
         val questsFile = folderHandle.child("Quests.json")
         if (questsFile.exists()) quests += createHashmap(json().fromJsonFile(Array<Quest>::class.java, questsFile))
 
@@ -338,10 +322,24 @@ class Ruleset {
                 // Append child policies of this branch
                 for (policy in branch.policies) {
                     policy.branch = branch
+                    policy.originRuleset = name
                     if (policy.requires == null) {
                         policy.requires = arrayListOf(branch.name)
                     }
+
+                    if (policy != branch.policies.last()) {
+                        // If mods override a previous policy's location, we don't want that policy to stick around,
+                        // because it leads to softlocks on the policy picker screen
+                        val conflictingLocationPolicy = policies.values.firstOrNull {
+                            it.branch.name == policy.branch.name
+                                && it.column == policy.column
+                                && it.row == policy.row
+                        }
+                        if (conflictingLocationPolicy != null)
+                            policies.remove(conflictingLocationPolicy.name)
+                    }
                     policies[policy.name] = policy
+
                 }
 
                 // Add a finisher
@@ -375,6 +373,7 @@ class Ruleset {
         val globalUniquesFile = folderHandle.child("GlobalUniques.json")
         if (globalUniquesFile.exists()) {
             globalUniques = json().fromJsonFile(GlobalUniques::class.java, globalUniquesFile)
+            globalUniques.originRuleset = name
         }
 
         val victoryTypesFile = folderHandle.child("VictoryTypes.json")
@@ -414,10 +413,20 @@ class Ruleset {
                         name = cityStateType.name
                         color = cityStateType.color
                         friendBonusUniques = ArrayList(cityStateType.friendBonusUniques.filter {
-                            RulesetValidator(this@Ruleset).checkUnique(Unique(it),false,"",UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific,UniqueTarget.CityState).isEmpty()
+                            RulesetValidator(this@Ruleset).checkUnique(
+                                Unique(it),
+                                false,
+                                cityStateType,
+                                UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific
+                            ).isEmpty()
                         })
                         allyBonusUniques = ArrayList(cityStateType.allyBonusUniques.filter {
-                            RulesetValidator(this@Ruleset).checkUnique(Unique(it),false,"",UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific,UniqueTarget.CityState).isEmpty()
+                            RulesetValidator(this@Ruleset).checkUnique(
+                                Unique(it),
+                                false,
+                                cityStateType,
+                                UniqueType.UniqueComplianceErrorSeverity.RulesetSpecific
+                            ).isEmpty()
                         })
                     }
         }
@@ -430,10 +439,11 @@ class Ruleset {
      *  */
     fun updateBuildingCosts() {
         for (building in buildings.values) {
-            if (building.cost == 0 && building.getMatchingUniques(UniqueType.Unbuildable).none { it.conditionals.isEmpty() }) {
+            if (building.cost == -1 && building.getMatchingUniques(UniqueType.Unbuildable).none { it.conditionals.isEmpty() }) {
                 val column = technologies[building.requiredTech]?.column
-                        ?: throw UncivShowableException("Building '[${building.name}]' is buildable and therefore must either have an explicit cost or reference an existing tech.")
-                building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
+                if (column != null) {
+                    building.cost = if (building.isAnyWonder()) column.wonderCost else column.buildingCost
+                }
             }
         }
     }
@@ -484,10 +494,11 @@ object RulesetCache : HashMap<String,Ruleset>() {
                 if (consoleMode) FileHandle(fileName)
                 else Gdx.files.internal(fileName)
             newRulesets[ruleset.fullName] = Ruleset().apply {
-                load(fileHandle)
                 name = ruleset.fullName
+                load(fileHandle)
             }
         }
+        this.putAll(newRulesets)
 
         val errorLines = ArrayList<String>()
         if (!noMods){
@@ -499,8 +510,8 @@ object RulesetCache : HashMap<String,Ruleset>() {
                 if (!modFolder.isDirectory) continue
                 try {
                     val modRuleset = Ruleset()
-                    modRuleset.load(modFolder.child("jsons"))
                     modRuleset.name = modFolder.name()
+                    modRuleset.load(modFolder.child("jsons"))
                     modRuleset.folderLocation = modFolder
                     newRulesets[modRuleset.name] = modRuleset
                     debug("Mod loaded successfully: %s", modRuleset.name)
