@@ -42,7 +42,7 @@ object UnitActions {
         val actionList = ArrayList<UnitAction>()
 
         // Determined by unit uniques
-        addTransformAction(unit, actionList)
+        addTransformActions(unit, actionList)
         addParadropAction(unit, actionList)
         addAirSweepAction(unit, actionList)
         addSetupAction(unit, actionList)
@@ -180,11 +180,11 @@ object UnitActions {
       */
     fun getFoundCityAction(unit: MapUnit, tile: Tile): UnitAction? {
         val unique = unit.getMatchingUniques(UniqueType.FoundCity)
-            .filter { it.conditionals.none { it.type == UniqueType.UnitActionExtraLimitedTimes } }
+            .filter { unique -> unique.conditionals.none { it.type == UniqueType.UnitActionExtraLimitedTimes } }
             .firstOrNull()
         if (unique == null || tile.isWater || tile.isImpassible()) return null
         // Spain should still be able to build Conquistadors in a one city challenge - but can't settle them
-        if (unit.civ.isOneCityChallenger() && unit.civ.hasEverOwnedOriginalCapital == true) return null
+        if (unit.civ.isOneCityChallenger() && unit.civ.hasEverOwnedOriginalCapital) return null
         if (usagesLeft(unit, unique)==0) return null
 
         if (unit.currentMovement <= 0 || !tile.canBeSettled())
@@ -201,6 +201,7 @@ object UnitActions {
 
             if (hasActionModifiers) activateSideEffects(unit, unique)
             else unit.destroy()
+            GUI.setUpdateWorldOnNextRender() // Set manually, since this could be triggered from the ConfirmPopup and not from the UnitActionsTable
         }
 
         if (unit.civ.playerType == PlayerType.AI)
@@ -308,48 +309,49 @@ object UnitActions {
         }
     }
 
-    private fun addTransformAction(
+    private fun addTransformActions(
         unit: MapUnit,
         actionList: ArrayList<UnitAction>
     ) {
-        val upgradeAction = getTransformAction(unit)
-        if (upgradeAction != null) actionList += upgradeAction
+        val upgradeAction = getTransformActions(unit)
+        actionList += upgradeAction
     }
 
     /**  */
-    private fun getTransformAction(
+    private fun getTransformActions(
         unit: MapUnit
-    ): ArrayList<UnitAction>? {
-        if (!unit.baseUnit().hasUnique(UniqueType.CanTransform)) return null // can't upgrade to anything
+    ): ArrayList<UnitAction> {
         val unitTile = unit.getTile()
         val civInfo = unit.civ
+        val stateForConditionals = StateForConditionals(unit = unit, civInfo = civInfo, tile = unitTile)
         val transformList = ArrayList<UnitAction>()
-        for (unique in unit.baseUnit().getMatchingUniques(UniqueType.CanTransform,
-            StateForConditionals(unit = unit, civInfo = civInfo, tile = unitTile))) {
-            val upgradedUnit = civInfo.getEquivalentUnit(unique.params[0])
-            // don't show if haven't researched/is obsolete
-            if (!unit.upgrade.canUpgrade(unitToUpgradeTo = upgradedUnit)) continue
+        for (unique in unit.baseUnit().getMatchingUniques(UniqueType.CanTransform, stateForConditionals)) {
+            val unitToTransformTo = civInfo.getEquivalentUnit(unique.params[0])
+
+            if (unitToTransformTo.getMatchingUniques(UniqueType.OnlyAvailableWhen)
+                    .any { !it.conditionalsApply(stateForConditionals) })
+                continue
 
             // Check _new_ resource requirements
             // Using Counter to aggregate is a bit exaggerated, but - respect the mad modder.
             val resourceRequirementsDelta = Counter<String>()
             for ((resource, amount) in unit.baseUnit().getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, -amount)
-            for ((resource, amount) in upgradedUnit.getResourceRequirementsPerTurn())
+            for ((resource, amount) in unitToTransformTo.getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, amount)
             val newResourceRequirementsString = resourceRequirementsDelta.entries
                 .filter { it.value > 0 }
                 .joinToString { "${it.value} {${it.key}}".tr() }
 
             val title = if (newResourceRequirementsString.isEmpty())
-                "Transform to [${upgradedUnit.name}]"
-            else "Transform to [${upgradedUnit.name}]\n([$newResourceRequirementsString])"
+                "Transform to [${unitToTransformTo.name}]"
+            else "Transform to [${unitToTransformTo.name}]\n([$newResourceRequirementsString])"
 
             transformList.add(UnitAction(UnitActionType.Transform,
                 title = title,
                 action = {
                     unit.destroy()
-                    val newUnit = civInfo.units.placeUnitNearTile(unitTile.position, upgradedUnit.name)
+                    val newUnit = civInfo.units.placeUnitNearTile(unitTile.position, unitToTransformTo.name)
 
                     /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
                      * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
@@ -362,9 +364,7 @@ object UnitActions {
                         newUnit.currentMovement = 0f
                     }
                 }.takeIf {
-                    unit.currentMovement > 0
-                            && !unit.isEmbarked()
-                            && unit.upgrade.canUpgrade(unitToUpgradeTo = upgradedUnit)
+                    unit.currentMovement > 0 && !unit.isEmbarked()
                 }
             ) )
         }
@@ -470,8 +470,7 @@ object UnitActions {
 
     fun getImprovementConstructionActions(unit: MapUnit, tile: Tile): ArrayList<UnitAction> {
         val finalActions = ArrayList<UnitAction>()
-        val uniquesToCheck = unit.getMatchingUniques(UniqueType.ConstructImprovementConsumingUnit) +
-                unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
+        val uniquesToCheck = unit.getMatchingUniques(UniqueType.ConstructImprovementInstantly)
         val civResources = unit.civ.getCivResourcesByName()
 
         for (unique in uniquesToCheck) {
@@ -501,8 +500,7 @@ object UnitActions {
                     // without this the world screen won't show the improvement because it isn't the 'last seen improvement'
                     unit.civ.cache.updateViewableTiles()
 
-                    if (unique.type == UniqueType.ConstructImprovementConsumingUnit) unit.consume()
-                    else activateSideEffects(unit, unique)
+                    activateSideEffects(unit, unique)
                 }.takeIf {
                     resourcesAvailable
                     && unit.currentMovement > 0f
@@ -636,6 +634,9 @@ object UnitActions {
         // If gifting to major civ they need to be friendly
         else if (!tile.isFriendlyTerritory(unit.civ)) return null
 
+        // Transported units can't be gifted
+        if (unit.isTransported) return null
+
         if (unit.currentMovement <= 0)
             return UnitAction(UnitActionType.GiftUnit, action = null)
 
@@ -747,7 +748,7 @@ object UnitActions {
     fun getMaxUsages(unit: MapUnit, actionUnique: Unique): Int? {
         val extraTimes = unit.getMatchingUniques(actionUnique.type!!)
             .filter { it.text.removeConditionals() == actionUnique.text.removeConditionals() }
-            .flatMap { it.conditionals.filter { it.type == UniqueType.UnitActionExtraLimitedTimes } }
+            .flatMap { unique -> unique.conditionals.filter { it.type == UniqueType.UnitActionExtraLimitedTimes } }
             .map { it.params[0].toInt() }
             .sum()
 
